/* 
 * Copyright (c) 2023 Emma Tebibyte
 * SPDX-License-Identifier: AGPL-3.0-or-later
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

use std::{
    collections::HashMap,
    io::{ BufRead, BufReader, Write },
    net::TcpStream,
    sync::{ Arc, Mutex },
};

struct Message {
    from: Option<String>,
    body: Option<String>,
}

pub fn handle_client(
    client: usize,
    clients: Arc<Mutex<HashMap<usize, TcpStream>>>
) {
    let mut reader = BufReader::new(
        clients.lock().unwrap().get(&client).unwrap().try_clone().unwrap()
    );

    let mut message = Message {
        from: None,
        body: None,
    };

    loop {
        let mut send = false;
        let mut buffer = String::new();

        if reader.read_line(&mut buffer).is_err() {
            println!(
                "{} disconnected.",
                message.from.unwrap_or_else(|| { format!("anon") }),
            );
            clients.lock().unwrap().remove(&client);
            break;
        }

        let words: Vec<&str> = buffer.split_whitespace().collect();
        if words.is_empty() {
            continue;
        }

        match words[0] {
            "FROM" => {
                message.from = Some(words[1..].join(" "));
            }
            "BODY" => {
                message.body = Some(words[1..].join(" "));
                send = true;
            }
            _ => {}
        }

        if send {
            let mut write_from = String::new();
            if message.from.is_some() {
                write_from = format!(
                    "FROM {}\n", message.from.clone().unwrap()
                );
            }

            for (id, mut stream) in clients
                .clone()
                .lock()
                .unwrap()
                .iter() {

                if id == &client { continue }

                if let Err(err) = stream.write_all(
                    format!(
                        "{}BODY {}\n",
                        write_from,
                        message.body.clone().unwrap(),
                    ).as_bytes()
                ) {
                    eprintln!("{}: {}", id, err);
                }
            }

            println!(
                "{}: {}",
                message.from.clone().unwrap_or_else(|| { format!("anon") }),
                message.body.take().unwrap(),
            );
        }
    }
}

